import scipy.stats as sc


def outliers(data, col, mode='zscore', threshold=3):

    '''OUTLIER FILTERING

    NOTE: this will automatically also drop nans from the dataset.

    '''

    # avoid destruction
    data = data.copy(deep=True)
    data = data[data[col].isna() == False]

    if mode == 'zscore':
        data['zscore'] = sc.zscore(data[col].astype(float))
        data = data[data.zscore < 3][data.zscore > -3].drop('zscore', axis=1)

    if mode == 'iqr':
        data = data[data[col] < sc.iqr(data[col]) * threshold]

    return data



import seaborn as sns
import matplotlib.pyplot as plt

from ..style.style import params
from ..style.titles import _titles
from ..style.formats import _thousand_sep
from ..style.template import _header, _footer
from ..utils.utils import _limiter, _scaler


def violin(data,
           x,
           y,
           hue=None,
           split=True,
           palette='default',
           style='astetik',
           dpi=72,
           title='',
           sub_title='',
           x_label='',
           y_label='',
           legend=True,
           x_scale='linear',
           y_scale='linear',
           x_limit=None,
           y_limit=None,
           outliers=False,
           save=False):

    '''VIOLIN PLOT

    A combination of kde style histogram and a box plotself.

    Inputs: 2 to 3
    Features: 1 continuous and 1 or 2 categoricals

    1. USE
    ======
    ast.violin(data=patients,
                x='insurance',
                y='age',
                hue='expired',
                y_limit=None)

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (categorical)

    y :: y-axis data (continuous or categorical)

    hue :: color highlight (categorical)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    split :: If False, 'hue' values will be separted in to
             two elements as opposed to shown together.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''
    if hue == None:
        legend = False

    # HEADER STARTS >>>
    palette = _header(palette, style, n_colors=2, dpi=dpi)
    # <<< HEADER ENDS

    p, ax = plt.subplots(figsize=(params()['fig_width'],
                                  params()['fig_height']))
    p = sns.violinplot(data=data,
                       x=x,
                       y=y,
                       hue=hue,
                       palette=palette,
                       split=split,
                       cut=3)

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)
    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y=y, x_limit=None, y_limit=y_limit)
    # <<< SCALING AND LIMITS ENDS

    # FOOTER STARTS >>>
    _thousand_sep(p, ax, data, x, y)
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, legend, 2, save)

    p.spines['bottom'].set_color('black')
    # <<< FOOTER ENDS



import pandas as pd

from matplotlib.dates import HourLocator, DayLocator, MonthLocator, YearLocator, MinuteLocator, SecondLocator
from matplotlib.dates import DateFormatter
from matplotlib import ticker


def _time_freq(time_data, divider1, divider2):

    out = (time_data.max() - time_data.min()).days
    print(out)
    out = round(out / divider2 / divider1)
    print(out)
    return out


def date_handler(time_data, ax, time_frame):

    ax.xaxis.set_major_locator(ticker.MaxNLocator(nbins=8))

    if time_frame == 'year':
        ax.xaxis.set_major_formatter(DateFormatter('%Y'))

    elif time_frame == 'month':
        ax.xaxis.set_major_locator(MonthLocator(bymonth=range(1, 13), interval=_time_freq(time_data, 8, 30)))
        ax.xaxis.set_major_formatter(DateFormatter('%Y-%m'))

    elif time_frame == 'day':
        ax.xaxis.set_major_locator(DayLocator(bymonthday=range(1, 32), interval=_time_freq(time_data, 8, 1)))
        ax.xaxis.set_major_formatter(DateFormatter('%Y-%m-%d'))

    elif time_frame == 'hour':
        ax.xaxis.set_major_locator(HourLocator(byhour=range(0, 24, 4)))
        ax.xaxis.set_major_formatter(DateFormatter('%H:%M'))

    elif time_frame == 'minute':
        ax.xaxis.set_major_locator(MinuteLocator(byminute=range(0, 60, 1), interval=60))
        ax.xaxis.set_major_formatter(DateFormatter('%H:%M'))

    elif time_frame == 'second':
        ax.xaxis.set_major_locator(SecondLocator(bysecond=range(0, 60, 1), interval=10))
        ax.xaxis.set_major_formatter(DateFormatter('%M:%S'))


def _generate_datetime(data, start, end, freq):

    '''DATETIME GENERATOR

    Takes in a series or dataframe, and based on the length creates
    a datetime series with a desired frequency. It's important to
    provide the start and end time precisely, together with the time
    unit, as otherwise there will be a mismatch of the valuesself.

    1.USE
    =====

    _generate_datetime(data, '1999-02-12','1999-03-05', 'hour')

    This example will generate datetimes with hourly frequency starting
    from midnight on the 12th of February (1999) up until midnight of
    5th of March.

    _generate_datetime(data, '1999-02-12-09:00','1999-03-05', 'hour')

    This example will do the same, but will start on 9:00 am instead.

    start :: the startime of the data (first observation timestamp)

    end :: the endtime of the data (last observation timestamp)

    freq :: This should be the frequency of observations in the dataset.
            Can be either pd.date_range frequency parameter, or:

            'year', 'month', 'day', 'hour', 'minute', 'second'
    '''

    if freq == 'year':
        freq = '365D'
    elif freq == 'month':
        freq = '30D'
    elif freq == 'day':
        freq = '1D'
    elif freq == 'hour':
        freq = '60Min'
    elif freq == 'minute':
        freq = '1Min'
    elif freq == 'second':
        freq = '1S'

    out = pd.Series(pd.date_range(start, end, freq=freq))

    if len(out) > len(data):
        print("Too many observation for the selected parameters.")
    elif len(out) < len(data):
        print("Not enough observations for the selected parameters.")

    return out



import matplotlib.pyplot as plt
import seaborn as sns

from ..style.formats import _thousand_sep
from ..style.style import params
from ..style.titles import _titles
from ..style.template import _header, _footer
from ..utils.utils import _limiter, _scaler
from ..style.sizer import _sizer


def scat(data,
         x,
         y,
         hue=None,
         size=None,
         palette='default',
         style='astetik',
         dpi=72,
         title='',
         sub_title='',
         x_label='',
         y_label='',
         legend=True,
         x_scale='linear',
         y_scale='linear',
         x_limit=None,
         y_limit=None,
         outliers=False,
         save=False):

    '''SCATTER PLOT

    Well actually, in comparison to a traditional scatter plot, there is
    are some limimations here. Observations may overlap, and sizing is
    possible. 'x' and 'hue' should not be continuous variables
    ('size' can be stepped). If you want basic scatter plot, use regs() and
    also see swarm() and strip() depending on your data.

    1. USE
    ======
    p = scat(data=df,
         x='Class',
         y='Fare',
         hue='Survived',
         size='Rand',
         palette='default',
         style='astetik')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data

    y :: y-axis data

    hue :: color highlight (categorical or boolean)

    size :: the size of the dots in the plot (continuous or stepped)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    None

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    # PLOT SPECIFIC START >>>
    if hue != None:
        n = len(data[hue].unique())
    else:
        n = 1
        legend = False

    if size == None:
        size = 8
    else:
        size = _sizer(data[size])
    # <<< PLOT SPECIFIC ENDS

    # HEADER STARTS >>>
    palette = _header(palette, style, n_colors=n, dpi=dpi)  # NOTE: y exception
    # <<< HEADER ENDS

    # # # # # # PLOT CODE STARTS # # # # # #
    p, ax = plt.subplots(figsize=(params()['fig_width'],
                                  params()['fig_height']))
    p = sns.stripplot(data=data,
                      x=x,
                      y=y,
                      hue=hue,
                      palette=palette,
                      linewidth=1,
                      size=size)
    # # # # # # PLOT CODE ENDS # # # # # #

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y=y, x_limit=x_limit, y_limit=y_limit)

    # START OF TITLES >>>
    _titles(title, sub_title=sub_title)
    _thousand_sep(p, ax, data, x, y)
    _footer(p, x_label, y_label, legend, n, save)



import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
from .exceptions import MissingParameter
from .transform import intervals


def _highlight_color(data,
                     color,
                     highlight_color,
                     highlight_mode,
                     highlight_value):

    '''Highlight Generator

    Takes as input colors and the highlight mode, and
    a highlight value, and then returns a color object
    to be used in a seaborn or matplotlib plot.

    '''
    try:
        if highlight_mode == '>=':
            colors = [highlight_color if _y >= highlight_value else color for _y in data]
        elif highlight_mode == '==':
            colors = [highlight_color if _y == highlight_value else color for _y in data]
        elif highlight_mode == '<=':
            colors = [highlight_color if _y <= highlight_value else color for _y in data]
        elif highlight_mode == '!=':
            colors = [highlight_color if _y != highlight_value else color for _y in data]
    except TypeError:
        raise MissingParameter("When highlight_mode is other than none you need to provide a highlight_value")
    return colors


def _sort_strings(data, column, sort):

    if sort == 'asc':
        asc = True
    elif sort == 'desc':
        asc = False
    data = data.sort_values(column, ascending=asc)

    return data


def table_prep(data, columns=''):

    """
      Data processor for table() function.

      You can call it separately as well and in
      return get a non-prettyfied summary table.

      Unless columns are defined, the three first
      columns are chosen by default.

      SYNTAX EXAMPLE:

      df['quality_score','influence_score','reach_score']

    """

    if data.shape[1] != 3:
        if len(columns) != 3:
            if data.shape[1] > 3:

                print("showing first three columns because no columns were \
                       specific / data had more than 3 columns")
                data = pd.DataFrame(data[data.columns[0:3]])

    if data.shape[1] < 3:

        print("You need at least 3 columns of data for this table")
        quit()

    if len(columns) == 3:
            data = data[columns]

    desc = pd.DataFrame({'sum': data.sum().astype('int'),
                         'median': data.median(),
                         'mean': data.mean(),
                         'std': data.std()})
    desc = desc.round(decimals=2)

    return desc


def _title_handling(p, data, title, sub_title, footnote, samplenote):

    '''TITLE HANDLER

    Takes care of the extremely painful task of positioning
    various titles and labels dynamically regardless of the plot.
    Or at least that's the idea...

    USE
    ===
    _title_handling(p, data, title, sub_title, samplenote, footnote)

    PARAMETERS
    ==========
    p :: the figure object
    data :: the data that is used in the plot
    title :: title string object or None
    sub_title :: sub_title string object or None
    footnote :: string object or None
    samplenote :: string object or None

    NOTE: At the moment works with one dimensional data.

    '''

    # handling title
    if title is not None:
        plt.title(title,
                  fontsize=28,
                  fontname='Verdana',
                  weight='bold',
                  verticalalignment='top',
                  horizontalalignment='left',
                  y=1.15,
                  x=.0,
                  color="grey")

    if sub_title is not None:
        plt.suptitle(sub_title,
                     fontsize=18,
                     fontname='Verdana',
                     verticalalignment='top',
                     horizontalalignment='left',
                     y=.9,
                     x=0.08,
                     color="grey")

    rel_bottom = 0 - (p.dataLim.bounds[3] / 7)

    if footnote is not None:
        plt.text(y=rel_bottom,
                 x=.2,
                 s="SOURCE: " + str(footnote),
                 color='grey',
                 verticalalignment='bottom',
                 horizontalalignment='left', )

    if samplenote is not None:
        plt.text(y=rel_bottom,
                 x=data.max(),
                 s="n = " + str(len(data)),
                 color='grey',
                 verticalalignment='bottom',
                 horizontalalignment='right')


def _scaler(p, x_scale=None, y_scale=None):

    if x_scale != None and x_scale != 'linear':
        p.set(xscale=x_scale)
    if y_scale != None and y_scale != 'linear':
        p.set(yscale=y_scale)


def _limiter(data=None, x=None, y=None, x_limit=None, y_limit=None):

    '''LIMITER (not for calling diretly )
    Handles x and y limits for all the plots. Can handle situations where
    there are more than one x as input.
    '''
    # SPECIAL CASE WITH MORE THAN ONE X
    if type(x) == type([]):
        max_list = []
        for i in range(len(x)):
            max_list.append(data[x[i]].max())
        x_max = max_list.index(max(max_list))

        min_list = []
        for i in range(len(x)):
            min_list.append(data[x[i]].min())
        x_min = min_list.index(min(min_list))

        x_max = data[x[x_max]].max()
        x_min = data[x[x_min]].min()

    # REGULAR CASE WITH JUST ONE X
    else:
        x_min = data[x].min()
        x_max = data[x].max()

    # HANDLING THE LIMITS STARTS
    if x_limit != None:
        # X-LIMS
        if x_limit != 'auto':
            if type(x_limit) == type(list):
                plt.xlim(x_limit[0], x_limit[1])
            else:
                plt.xlim(x_limit,)
        elif x_limit == 'auto':
            plt.xlim(x_min, x_max * 1.1)

    if y_limit != None:
        # SPECIAL CASES LIKE LINE PLOTS
        if y == '_R_E_S_':
            y_min = x_min
            y_max = x_max
        else:
            y_min = data[y].min()
            y_max = data[y].max()

        # Y-LIMS
        if y_limit != 'auto':
            if type(y_limit) == type(list):
                plt.ylim(y_limit[0], y_limit[1])
            else:
                plt.ylim(y_limit,)
        elif y_limit == 'auto':
            plt.ylim(y_min, y_max * 1.1)


def _n_decider(y):

    '''PRODUCE N-VALUE FOR _HEADER

    Takes in 'y' and based on the format
    returns the 'n' value that is used
    by the color picker.

    '''

    # no input value
    if type(y) == type(None):
        n = 10

    # equal to uniques
    elif type(y) == type(1):
        n = y

    # is a list or series
    elif type(y) == type(pd.Series()):
        n = 1

    elif type(y) == type([]) or type(y) == type(np.array(0)):
        n = len(y)

    elif type(y) == type(''):
        n = 1

    # equal to input value
    else:
        n = n

    return n


def _check_type(data):

    '''DTYPE CHECKER

    Checks if the first value in a dataset after
    dropping NAs is float or int.

    '''

    data_type = type(data.dropna().values[0])

    if np.issubdtype(data_type, int) == True:
        data_type = 'int'
    elif np.issubdtype(data_type, float) == True:
        data_type == 'float'

    return data_type


def multicol_transform(transform, data, x=None, y=None, func=None, freq=None):

    out = pd.DataFrame()

    if type(x) != type([]):
        x = [x]

    for col in x:
        if transform == 'interval':
            temp = intervals(data=data, x=col, dt_col=y, mode=func, freq=freq)
        out[col] = temp[col]
    out[y] = temp[y]

    return out


def factorplot_sizing(data, width=9, thickness=3, auto=False):

    '''COMPUTE SIZE AND ASPECT

    This is for the sole purpose of standardizing
    the factorplot sizing so that bar thickness
    is same regardless of the number of bars in the figure.
    '''

    if auto == True:
        items = len(data.value_counts())
    else:
        items = len(data)

    # compute the values
    value = items + 2.5
    size = value / thickness
    aspect = (width / size)

    return size, aspect



import seaborn as sns
import matplotlib.pyplot as plt

from ..style.template import _header
from ..utils.transform import _groupby


def multikde(data,
             x,
             label_col,
             sort=None,
             transform_func=False,
             palette='default',
             style='astetik',
             dpi=72,
             title='',  # not working here
             sub_title='',  # not working here
             x_label='',  # not working here
             y_label='',  # not working here
             legend=False,  # not working here
             x_scale='linear',  # not working here
             y_scale='linear',  # not working here
             x_limit='auto',  # not working here
             y_limit='auto',  # not working here
             save=False):

    '''MULTI KDE

    Creates a plot with several kde style histograms
    one on top of each other. Great for comparing categorical
    features against continuous features.

    Note that this does not work well with data where the
    difference between min and max values is big.

    Inputs: 2
    Features: 1 continuous and 1 categorical

    1. USE
    ======
    ast.multikde(data=patients,
             x='age',
             label_col='insurance')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (continuous)

    y :: x-axis overlap data (continuous)

    label_col :: the column with the label values

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    sort :: either True or False for ascending sort based on the
            x-axis data.

    transform_func :: If not false, the selected function such as
                      'mean' will be used to group by the label_col.
                      Available functions:
                        - 'median'
                        - 'mean'
                        - 'first'
                        - 'last',
                        - 'std',
                        - 'mode',
                        - 'max',
                        - 'min',
                        - 'sum',
                        - 'random'

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    if transform_func != False:
        data = _groupby(data, label_col, transform_func)

    palette = _header(palette,
                      style,
                      n_colors=1,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS

    sns.set(style="white", rc={"axes.facecolor": (0, 0, 0, 0)})
    g = sns.FacetGrid(data,
                      row=label_col,
                      hue=label_col,
                      aspect=15,
                      size=1,
                      palette=palette)

    g.map(sns.kdeplot, x, clip_on=False, shade=True, alpha=1, lw=1.5, bw=.2)
    g.map(plt.axhline, y=0, lw=2, clip_on=False)

    # Define and use a simple function to label the plot in axes coordinates
    def label(x, color, label):
        ax = plt.gca()
        ax.text(0, .2,
                label,
                fontweight="bold",
                fontsize=14,
                color=color,
                ha="left",
                va="center",
                transform=ax.transAxes)

    g.map(label, x)

    # Set the subplots to overlap
    g.fig.subplots_adjust(hspace=-.25)

    # Remove axes details that don't play will with overlap
    g.set_titles("")
    g.set(yticks=[])
    g.despine(bottom=True, left=True);



import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches

from ..style.titles import _titles
from ..style.template import _header
from ..utils.transform import _groupby
from ..style.formats import _thousand_sep


def overlap(data,
            x,
            y,
            label_col,
            sort=None,
            limit=None,
            transform_func=False,
            palette='default',
            style='astetik',
            dpi=72,
            title='',
            sub_title='',
            x_label='',
            y_label='',
            legend=True,
            x_scale='linear',
            y_scale='linear',
            x_limit=None,
            y_limit=None,
            save=False):

    '''OVERLAP BAR PLOT

    Useful for the cases where you have a categorical
    feature, and then you want to compare two overlapping
    continuous features (e.g. all days and rainy days) with
    each other per category. Each category will have its own
    bar, where the 'x' and 'y' features will be overlapping.

    Inputs: 3
    Features: 2 continuous and 1 categorical

    NOTE: 'y' should be a subset of 'x'.

    1. USE
    ======
    ast.overlap(data=patients,
                x='hospital_days',
                y='icu_days',
                label_col='insurance',
                sort=False,
                transform=True,
                transform_func='sum',
                palette='colorblind')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (continuous)
    y :: x-axis overlap data (continuous)
    label_col :: the column with the label values

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    sort :: either True or False for ascending sort based on the
            x-axis data.

    limit :: limit the number of items to be shown

    transform_func :: If not False, the selected function such as
                      'mean' will be used to group by the label_col.
                      Available functions:
                        - 'median'
                        - 'mean'
                        - 'first'
                        - 'last',
                        - 'std',
                        - 'mode',
                        - 'max',
                        - 'min',
                        - 'sum',
                        - 'random'

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    if transform_func != False:
        data = _groupby(data, label_col, transform_func)

    if sort != None:
        data = data.sort_values(x, ascending=sort)

    fig_height = len(data[label_col].unique()) * 0.6
    p, ax = plt.subplots(figsize=(6, fig_height))

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=2,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS

    # # # # PLOT STARTS # # # #

    sns.barplot(data=data,
                x=x,
                y=label_col,
                orient='h',
                color=palette[0])

    sns.barplot(data=data,
                x=y,
                y=label_col,
                orient='h',
                color=palette[1])

    # # # # PLOT ENDS # # # #
    if legend != False:
        x_patch = mpatches.Patch(color=palette[0], label=x)
        y_patch = mpatches.Patch(color=palette[1], label=y)
        ax.legend(handles=[x_patch, y_patch], ncol=1, loc="upper right", frameon=True)

    ax.set(ylabel=y_label, xlabel=x_label)
    sns.despine(bottom=True)
    ax.xaxis.set_major_locator(plt.MaxNLocator(5))

    _thousand_sep(p, ax, data, x, y)
    if len(title) + len(sub_title) < 0:
        _titles(title, sub_title=sub_title)



import seaborn as sns
import matplotlib.pyplot as plt

from ..style.formats import _thousand_sep
from ..style.style import params
from ..style.titles import _titles
from ..style.template import _header, _footer
from ..utils.utils import _limiter, _scaler


def kde(data,
        x,
        y=None,
        cumulative=False,
        palette='default',
        style='astetik',
        dpi=72,
        title='',
        sub_title='',
        x_label='',
        y_label='',
        legend=False,
        x_scale='linear',
        y_scale='linear',
        x_limit=None,
        y_limit=None,
        save=False):

    '''KDE PLOT

    Create a single Kernel Estimation Density (KDE) style
    histogram.

    Inputs: 1 or 2
    Features: At least 1 continuous

    1. USE
    ======
    ast.kde(data=df
            x='Fare',
            palette='blue_to_red')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (continuous)

    y :: x-axis overlap data (continuous or categorical)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    cumulative :: If True, showing the cumulative distribution.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    if y != None:
        n = 2
        data2 = data[y]
    else:
        n = 1
        data2 = None

    # HEADER
    palette = _header(palette, style, n_colors=n, dpi=dpi)

    p, ax = plt.subplots(figsize=(params()['fig_width'],
                                  params()['fig_height']))

    p = sns.kdeplot(data=data[x],
                    data2=data2,
                    shade=True,
                    cut=5,
                    shade_lowest=False,
                    color=palette[0],
                    legend=legend,
                    cumulative=cumulative,
                    kernel='gau',
                    bw='scott')

    # SCALING AND LIMITS
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y=y, x_limit=x_limit, y_limit=y_limit)

    # FOOTER
    _thousand_sep(p, ax, data, x, y)
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, save=save)



import matplotlib.pyplot as plt
from matplotlib import ticker

from ..style.titles import _titles
from ..style.template import _header, _footer

from sklearn.metrics import roc_curve


def roc(y_pred,
        y_true,
        interval=False,
        interval_func=None,
        time_frame=None,
        dropna=False,
        median_line=False,
        drawstyle='default',
        linestyle='solid',
        legend_labels=None,
        palette='default',
        style='astetik',
        dpi=72,
        title='',
        sub_title='',
        x_label='',
        y_label='',
        legend=False,
        x_scale='linear',
        y_scale=None,
        x_limit=None,
        y_limit=None,
        save=False):

    '''TIMESERIES LINE PLOT

    A line plot for one or more columns all with a comparable
    value, in a time sequence. IF 'x' is None, all columns except
    'y' will be included.

    1.USE
    =====
    line_plot(data=ldata,
              x='value',
              linestyle='dashdot',
              palette='colorblind',
              title="The main title comes here",
              sub_title="Suptibtle comes here")

    Inputs: 1 or more continuous and an optional timestamp
    Features: Continuous and optional datetime format

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: one or more columns of data

    y :: a single timeseries column (no need to be dt)
         and if y is not defined, then a sequence will
         be automatically generated as time labels.

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    interval :: If not False, should be number of minutes per
                 sample as int or one of the presets:
                 - 'quarter'
                 - 'half',
                 - 'full' (days)
                 - 'week'
                 - 'month' (30 days)
                 - 'year'.

    interval_func :: The grouping by function that will be used:

                        'median', 'mean', 'mode',
                        'first', 'last', 'std', 'mode'
                        'max', 'min', 'sum', 'random',
                        or 'freq'

    time_frame :: the time frame to be used for x-axis labels:

                  'year', 'month', 'day', 'hour', 'minute', 'second'

    median_line :: If True, a median line will be drawn

    drawstyle :: 'default', 'steps', 'steps-pre','steps-mid' or 'steps-post'

    linestyle :: 'solid', 'dashed', 'dashdot' , 'dotted'

    markerstyle :: ".", ",", "o", "+", "x", "|", "_", "^", "v"

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
               'default'
               'colorblind'
               'blue_to_red'
               'blue_to_green'
               'red_to_green'
               'green_to_red'
               'violet_to_blue'
               'brown_to_green'
               'green_to_marine'

               Or use any cmap, seaborn or matplotlib
               color or palette code, or hex value.

    style :: Use one of the three core styles:
               'astetik'     # white
               '538'         # grey
               'solarized'   # sepia

             Or alternatively use any matplotlib or seaborn
             style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    # START OF PLOT SPECIFIC >>>

    fpr, tpr, thresholds = roc_curve(y_pred, y_true)

    # START OF HEADER >>>
    palette = _header(palette, style, n_colors=1, dpi=dpi)
    # <<< END OF HEADER

    p, ax = plt.subplots(figsize=(7, 7))

    plt.plot([0, 1], [0, 1], linewidth=2, color='#1B2F33', linestyle='dashed')
    plt.plot(fpr, tpr,
             linewidth=3,
             color=palette,
             linestyle=linestyle,
             drawstyle=drawstyle,
             c=palette,
             rasterized=True,
             aa=True,
             alpha=1)

    # HEADER
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, save=save)

    plt.axis([0, 1, 0, 1])

    if legend != False:
        if legend_labels != None:
            x = legend_labels
        plt.legend(x,
                   framealpha=0.0,
                   fontsize=14,
                   loc=1,
                   ncol=1,
                   bbox_to_anchor=(1.35, 1.0))

    ax.xaxis.set_major_locator(ticker.MaxNLocator(nbins=6))
    ax.yaxis.set_major_locator(ticker.MaxNLocator(nbins=6))



import seaborn as sns

from ..style.template import _header, _footer


def grid(data,
         x,
         y,
         col=None,
         hue=None,
         col_wrap=4,
         palette='default',
         style='astetik',
         dpi=72,
         title='',
         sub_title='',
         x_label='',
         y_label='',
         legend=True,
         x_scale='linear',
         y_scale='linear',
         x_limit=None,
         y_limit=None,
         save=False):

    '''THE GRID

    The grid provides an overview of 4 features simultanously by
    drawing a grid of scatter plots.

    Inputs: 4
    Features: Ideally two continuous, and two categorical, but will
              also work with just one continuous and two categoricals.

    1. USE
    ======
    ast.grid(data=new_patients.head(1000),
              x='icu_stays',
              y='hospital_days',
              col='religion',
              palette='default',
              col_wrap=4);

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (continuous or categorical)

    y :: y-axis data (continuous)

    hue :: color highlight (categorical)

    col :: the side-by-side plot comparison feature

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    col_wrap :: the number of plots to show per row

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    data = data.copy(deep=True)

    if hue != None:
        n_colors = len(data[hue].unique())
    else:
        n_colors = 1

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=n_colors,
                      dpi=72,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS

    p = sns.factorplot(data=data,
                       x=x,
                       y=y,
                       col=col,
                       hue=hue,
                       palette=palette,
                       col_wrap=4,
                       kind='strip',
                       size=3)

    # FOOTER STARTS >>>
    _footer(p, x_label, y_label, save=save)

    sns.despine(bottom=True, left=True)
    p.set(xticklabels=[])



import pandas as pd

import pkg_resources

countries = pkg_resources.resource_filename(__name__, '../extras/countries.csv')
countries = pd.read_csv(countries)


def code_to_country(code):

    out = countries.set_index('alpha-3')
    out = out['name'].to_dict()

    try:
        return out[code]
    except:
        return 'UNKNOWN_CODE'


def country_to_code(country):

    out = countries.set_index('name')
    out = out['alpha-3'].to_dict()

    try:
        return out[country]
    except:
        return 'UNKNOWN_COUNTRY'



class MissingParameter(Exception):
    """This error is raised when something goes wrong due to missing parameter"""

class MissingLabel(Exception):
    """This error is raised when something goes wrong due to missing labels"""

class MissingImport(Exception):
    """This error is raised when an import is missing but fail is not needed"""



import seaborn as sns
import pandas as pd

from ..style.template import _header, _footer
from ..utils.utils import _scaler
from ..utils.utils import factorplot_sizing
from ..style.titles import _titles


def bar(data,
        x=None,
        y=None,
        sort=None,
        order=None,
        multi_color=False,
        group_by_y=True,
        orient='h',
        aspect=1,
        size=5,
        palette='default',
        alpha=1,
        style='astetik',
        dpi=72,
        title='',
        sub_title='',
        x_label='',
        y_label='',
        legend=True,
        x_scale='linear',
        y_scale='linear',
        x_limit=None,
        y_limit=None,
        save=False):

    '''BAR PLOT

    A 1-dimensional bar graph for the case where there is a single
    value per label. Accepts either dataframe or series. If series,
    then labels will come from index.

    Inputs: 1 (Series), 2(DataFrame)

    1. USE
    ======
    ast.bar(data=patients,
            x='icu_days',
            y='insurance')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data (single value per label)

    y :: y-axis data (labels)

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    sort :: If True, values will be sorted ascending, if False descending.

    multi_color :: If True, label values will be used for hue.

    group_by_y :: If set to False, duplicate values will not be merged.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the astetik palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'
    '''
    # convert series to dataframe
    if x == None and y == None:
        data = data.copy(deep=True)
        data = pd.DataFrame(data)
        data.reset_index(inplace=True)
        data.columns = ['x', 'y']
        x = 'y'
        y = 'x'

    #size, aspect = factorplot_sizing(data[x])

    # merge duplicate items
    if group_by_y == True:
        data = data.groupby(y).sum().reset_index()
        data.columns = [y, x]

    if sort is not None:
        order = data[x].sort_values(ascending=sort)

    if multi_color == True:
        n_colors = len(data[x].unique())
    else:
        n_colors = 1

    # HEADER STARTS >>>
    palette = _header(palette,
                      style,
                      n_colors=n_colors,
                      dpi=dpi,
                      fig_height=None,
                      fig_width=None)
    # <<< HEADER ENDS
    p = sns.catplot(data=data,
                       x=x,
                       y=y,
                       palette=palette,
                       orient=orient,
                       aspect=aspect,
                       height=size,
                       alpha=alpha,
                       order=order,
                       kind='bar')

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    # FOOTER STARTS >>>
    _titles(title, sub_title=sub_title)
    _footer(p, x_label, y_label, save=save)

    if data[x].min() < 0:
        sns.despine(left=True)



import math
import pandas as pd
import numpy as np


def groupby_func(data, func):

    if func == 'median':
        out = data.median()
    elif func == 'mean':
        out = data.mean()
    elif func == 'first':
        out = data.first()
    elif func == 'last':
        out = data.last()
    elif func == 'std':
        out = data.std()
    elif func == 'mode':
        out = data.mode()
    elif func == 'max':
        out = data.max()
    elif func == 'min':
        out = data.min()
    elif func == 'sum':
        out = data.sum()
    elif func == 'random':
        out = data.agg(np.random.choice)
    elif func == 'freq':
        out = data.agg(lambda x: x.value_counts().index[0])

    try:
        out = out.reset_index()
    except UnboundLocalError:
        raise ValueError('No group_by argument provided to groupby_func()')

    return out


def rescaler(values, scale=1, to_int=False):

    '''MinMax Rescaler
    WHAT: rescales a set of values on to a fixed scale.
    HOW: max_rescale([10,6,2],1)
    INPUT: an array, list or Series
    OUTPUT: an array with rescaled values.
    '''

    multiplier = scale / np.array(values).max().astype(float)
    new_shape = np.array(values) * multiplier

    if to_int is True:

        l = []

        for value in new_shape:
            l.append(int(math.ceil(value)))

        return np.array(l)

    else:
        return new_shape


def intervals(data, x, dt_col, mode='first', freq=60):

    """BREAK TIMESERIES TO INTERVALS
    USE
    ===
    test = intervals(time_data, 'value','time_stamp', mode='min')

    PARAMETERS
    ----------
    data :: a dataframe with the values and a datetime column
    x :: the column with the value
    dt_col :: the column with the datetime values
    mode :: 'median', 'mean', 'mode', 'first', 'last', 'std', 'mode'
            'max', 'min', 'sum', 'random', 'freq'
    freq :: number of minutes per sample as int or a string 'quarter', 'half',
            'full' (days), 'week', 'month' (30 days), 'year'.

    """

    if type(freq) == type('s'):
        if freq == 'quarter':
            freq = 240
        elif freq == 'half':
            freq = 720
        elif freq == 'full':
            freq = 1440
        elif freq == 'week':
            freq = 10080
        elif freq == 'month':
            freq = 32400
        elif freq == 'year':
            freq = 525600

    freq = str(freq) + "Min"

    time_temp = data[[x, dt_col]].set_index(dt_col)
    time_temp = time_temp.groupby(pd.Grouper(freq=freq, label='right'))

    return groupby_func(data=time_temp, func=mode)


def equal_samples(data, col, sample_size):

    '''Resamples based on unique labels in a column.

    Results in a dataframe with equal number of rows
    per label in a given column.
    '''
    new_data = pd.DataFrame()

    for col_label in data[col].unique():
        sample = data[data[col] == col_label].sample(sample_size)
        new_data = new_data.append(sample)
    out = new_data

    return out


def mean_zero(data, retain=None):

    '''ZERO MEAN SCALER

    Takes in a dataframe or series, and rescales features
    so that mean for each feature becomes 0 and standard
    deviation becomes 1.

    USE
    ===
    mean_zero(df[['Age','Fare','Pclass']],retain='Pclass')

    PARAMETERS
    ==========
    data :: pandas dataframe or series object
    retain :: if some column should be excluded from rescaling

    '''
    # avoiding transformation of y, labels, etc

    data = data.copy(deep=True)

    try:
        col_list = list(data.columns)
    except AttributeError:
        col_list = list(pd.DataFrame(data.columns))

    if retain is not None:
        col_list.remove(retain)

    for col in col_list:

        # storing the temp values
        data_mean = data[col].mean(axis=0)
        data_std = data[col].std(axis=0)

        # transforming the data
        col_data = data[col] - data_mean
        data[col] = col_data / data_std

    return data


def _groupby(data, by, func):

    '''GROUP BY

    Takes in a dataframe and returns it in a grouped by format.

    PARAMETERS
    ----------
    data :: a pandas dataframe
    by :: the column by which the grouping is done
    func ::


    '''

    temp = data.groupby(by)

    return groupby_func(data=temp, func=func)


def _boolcols_to_cat(data, labels, other_label='NA'):

    '''CONVERT BOOLEAN COLS TO CATEGORICAL

    Returns a single categorical label seqeuence
    that is produced from 2 or more boolean columns.

    '''

    c = len(data)
    l = ['~'] * c

    for i in range(c):
        for label in labels:
            if data[label][i] == True:
                l[i] = label
                break

    l = [label.replace('~', other_label) for label in l]

    return l



#! /usr/bin/env python
#
import os
# temporarily redirect config directory to prevent matplotlib importing
# testing that for writeable directory which results in sandbox error in
# certain easy_install versions
os.environ["MPLCONFIGDIR"] = "."

DESCRIPTION = "Astetik data visualization and reporting library"
LONG_DESCRIPTION = """\

astetik provides a very high level overlay on Seaborn and matplotlib.
It is a data visualization library for data exploration, and for
telling captivating stories with data. Unlike any other visualization
library, astetik is specifically made for day-to-day use by data
scientists and reduces learning curve compared to similar solutions
significantly.

"""

DISTNAME = 'astetik'
MAINTAINER = 'Mikko Kotila'
MAINTAINER_EMAIL = 'mailme@mikkokotila.com'
URL = 'http://mikkokotila.com'
LICENSE = 'MIT'
DOWNLOAD_URL = 'https://github.com/mikkokotila/pretty'
VERSION = '1.13'

try:
    from setuptools import setup
    _has_setuptools = True
except ImportError:
    from distutils.core import setup

def check_dependencies():
    install_requires = []

    try:
        import seaborn
    except ImportError:
        install_requires.append('seaborn')
    try:
        import wrangle
    except ImportError:
        install_requires.append('wrangle')
    try:
        import numpy
    except ImportError:
        install_requires.append('numpy')
    try:
        import pandas
    except ImportError:
        install_requires.append('pandas')
    try:
        import IPython
    except ImportError:
        install_requires.append('IPython')
    try:
        import patsy
    except ImportError:
        install_requires.append('patsy')
    try:
        import geonamescache
    except ImportError:
        install_requires.append('geonamescache')
    try:
        import statsmodels
    except ImportError:
        install_requires.append('statsmodels')
    try:
        import sklearn
    except ImportError:
        install_requires.append('sklearn')

    return install_requires

if __name__ == "__main__":

    install_requires = check_dependencies()

    setup(name=DISTNAME,
        author=MAINTAINER,
        author_email=MAINTAINER_EMAIL,
        maintainer=MAINTAINER,
        maintainer_email=MAINTAINER_EMAIL,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        license=LICENSE,
        url=URL,
        version=VERSION,
        download_url=DOWNLOAD_URL,
        include_package_data=True,
        package_data={'astetik.extras': ['*']},
        install_requires=install_requires,
        packages=['astetik',
                  'astetik.utils',
                  'astetik.tables',
                  'astetik.style',
                  'astetik.plots',
                  'astetik.extras'],
        classifiers=[
                     'Intended Audience :: Science/Research',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3.4',
                     'Programming Language :: Python :: 3.5',
                     'Programming Language :: Python :: 3.6',
                     'License :: OSI Approved :: MIT License',
                     'Topic :: Scientific/Engineering :: Visualization',
                     'Topic :: Multimedia :: Graphics',
                     'Operating System :: POSIX',
                     'Operating System :: Unix',
                     'Operating System :: MacOS'],
          )



import matplotlib.pyplot as plt
import seaborn as sns

from ..style.formats import _thousand_sep
from ..style.style import params
from ..style.titles import _titles
from ..style.template import _header, _footer
from ..utils.utils import _limiter, _scaler


def regs(data,
         x,
         y,
         marker='x',
         fit_reg=False,
         draw_scatter=True,
         logres=False,
         palette='default',
         style='astetik',
         dpi=72,
         title='',
         sub_title='',
         x_label='',
         y_label='',
         legend=True,
         x_scale='linear',
         y_scale='linear',
         x_limit=None,
         y_limit=None,
         outliers=False,
         save=False):

    '''REGRESSION SCATTER PLOT

    This can be used as a standard two variable scatter plot
    where both variables are continuous. If x is categorical,
    use scat() instead or swarm(), depending on your need.
    Here you can also choose logres as True in case one of your
    variables is a binary categorical.

    1. USE
    ======
    p = regs(data=df,
         x='Age',
         y='Fare',
         palette='default',
         style='astetik')

    2. PARAMETERS
    =============
    2.1 INPUT PARAMETERS
    --------------------
    data :: pandas dataframe

    x :: x-axis data

    y :: y-axis data

    --------------------
    2.2. PLOT PARAMETERS
    --------------------
    marker :: By default is 'x' but you can use any matplotlib supported

    fit_reg :: If the regression fit should be drawn

    draw_scatter :: if the scatter should be drawn

    logres :: for logistic regression when one feature is binary categorical.

    ----------------------
    2.3. COMMON PARAMETERS
    ----------------------
    palette :: One of the hand-crafted palettes:
                'default'
                'colorblind'
                'blue_to_red'
                'blue_to_green'
                'red_to_green'
                'green_to_red'
                'violet_to_blue'
                'brown_to_green'
                'green_to_marine'

                Or use any cmap, seaborn or matplotlib
                color or palette code, or hex value.

    style :: Use one of the three core styles:
                'astetik'     # white
                '538'         # grey
                'solarized'   # sepia

              Or alternatively use any matplotlib or seaborn
              style definition.

    dpi :: the resolution of the plot (int value)

    title :: the title of the plot (string value)

    sub_title :: a secondary title to be shown below the title

    x_label :: string value for x-axis label

    y_label :: string value for y-axis label

    x_scale :: 'linear' or 'log' or 'symlog'

    y_scale :: 'linear' or 'log' or 'symlog'

    x_limit :: int or list with two ints

    y_limit :: int or list with two ints

    outliers :: Remove outliers using either 'zscore' or 'iqr'

    '''

    # HEADER STARTS >>>
    palette = _header(palette, style, n_colors=1, dpi=dpi)  # NOTE: y exception
    # <<< HEADER ENDS

    # # # # # # PLOT CODE STARTS # # # # # #
    p, ax = plt.subplots(figsize=(params()['fig_width'],
                                  params()['fig_height']))

    sns.regplot(data=data,
                x=x, y=y,
                fit_reg=fit_reg,
                scatter=draw_scatter,
                color=palette[0],
                marker=marker,
                logistic=logres)
    # # # # # # PLOT CODE ENDS # # # # # #

    # SCALING AND LIMITS STARTS >>>
    if x_scale != 'linear' or y_scale != 'linear':
        _scaler(p, x_scale, y_scale)

    if x_limit != None or y_limit != None:
        _limiter(data=data, x=x, y=y, x_limit=x_limit, y_limit=y_limit)

    # START OF TITLES >>>
    _titles(title, sub_title=sub_title)

    _thousand_sep(p, ax, data, x, y)
    _footer(p=p, xlabel=x_label, ylabel=y_label, legend=False, n=1, save=save)

    ax.xaxis.set_major_locator(plt.MaxNLocator(5))



def default_colors():

    return '#696969'


def params():

    param_dic = {'fig_height': 6.6,
                 'fig_width': 10}

    return param_dic

def styles(dpi):

    default_color = default_colors()

    label_sizes = 12

    style_dic = {
            # FONT
            'font.size': 14,
            'font.family': 'Verdana, Geneva, sans-serif',
            'font.stretch': 'semi-expanded',
            'font.style': 'normal',
            'font.variant': 'normal',
            'font.weight': 'normal',

            # AXES
            'axes.linewidth': 1,        # spine thickness
            'axes.xmargin': 0.05,       # spine distance from grid
            'axes.ymargin': 0.05,

                # AXES TITLE
                'axes.titlesize': 11.0,
                'axes.titlepad': 10.0,
                'axes.titleweight': 'normal',
                'axes.labelpad': 15,

            # FIGURE
            'figure.dpi': dpi,

            # GRID
            'axes.grid': True,
            'grid.alpha': .1,
            'grid.color': 'grey',
            'grid.linewidth': 1,
            'grid.linestyle': '-',

            # LINES
            #'lines.antialiased': True,
            'lines.linestyle': '-',
            'lines.linewidth': 0.75,           # plot element outlines
            'lines.color': '#000000',

            # LEGEND
            'legend.framealpha': 1,
            'legend.frameon': False,
            'legend.handleheight': 2.5,
            'legend.handlelength': 2.5,
            'legend.loc': 'upper right',
            'legend.labelspacing': 0.8,
            'legend.borderpad': 0.5,
            'legend.columnspacing': 2.0,

            # SAVEFIG
            'savefig.bbox': None,
            'savefig.directory': '~',
            'savefig.dpi': 'figure',
            'savefig.edgecolor': 'white',
            'savefig.format': 'png',
            'savefig.orientation': 'portrait',
            'savefig.pad_inches': 0.1,

            # # # TICKS START

            # X-TICK
            'xtick.alignment': 'center',
            'xtick.bottom': True,
            'xtick.color': default_color,
            'xtick.direction': 'out',
            'xtick.labelbottom': True,
            'xtick.labelsize': label_sizes,
            'xtick.labeltop': True,

                # MAJOR X-TICK
                'xtick.major.bottom': True,
                'xtick.major.pad': 7.0,
                'xtick.major.size': 0.0,
                'xtick.major.top': False,
                'xtick.major.width': 0.0,

                # MINOR X-TICK
                'xtick.minor.bottom': True,
                'xtick.minor.pad': 3.4,
                'xtick.minor.size': 0.0,
                'xtick.minor.top': True,
                'xtick.minor.visible': False,
                'xtick.minor.width': 0.5,
                'xtick.top': False,

            # Y-TICK
            'ytick.alignment': 'center_baseline',
            'ytick.color': default_color,
            'ytick.direction': 'out',
            'ytick.labelleft': True,
            'ytick.labelright': False,
            'ytick.labelsize': label_sizes,
            'ytick.left': True,

                # MAJOR Y-TICK
                'ytick.major.left': True,
                'ytick.major.pad': 15.0,
                'ytick.major.right': False,
                'ytick.major.size': 0.0,
                'ytick.major.width': 11.0,

                # MINOR Y-TICK
                'ytick.minor.left': False,
                'ytick.minor.pad': 3.4,
                'ytick.minor.right': False,
                'ytick.minor.size': 6.0,
                'ytick.minor.visible': False,
                'ytick.minor.width': 1,
                'ytick.right': False,

            # BOXPLOT SPECIFIC
            'boxplot.notch': False,
            'boxplot.patchartist': False,
            'boxplot.showbox': True,
            'boxplot.showcaps': False,
            'boxplot.showfliers': False,
            'boxplot.showmeans': False,
            'boxplot.vertical': False,

    }

    return style_dic



import numpy as np


def randomcolor():

    ''' PICKS COLORS RANDOMLY

    '''

    colors = []
    for i in range(20):
        colors.append(list((np.random.randint(0, 255, 3) / 255)))

    return colors
